/* Copyright (C) 2002 John Edwards <john.edwards33@ntlworld.com>
   File: main.c

   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
   License as published by the Free Software Foundation; either
   version 2.1 of the License, or (at your option) any later version.
   
   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Lesser General Public License for more details.
   
   You should have received a copy of the GNU Lesser General Public
   License along with this library; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

 */

#include <windows.h>
#include <shellapi.h>
#include <string.h>
#include <stdio.h>
#include <commctrl.h>
#include <shlobj.h>	// for Output Directory options

#include "resource.h"
#include "encthread.h"
#include "misc.h"
#include "encode.h"

#define LOSHORT(l)           ((SHORT)(l))
#define HISHORT(l)           ((SHORT)(((DWORD)(l) >> 16) & 0xFFFF))

#define INI_FILE "speexdrop.ini"

#define CREATEFONT(sz) \
	CreateFont((sz), 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, \
		VARIABLE_PITCH | FF_SWISS, "")
   
char         *fileName,
             config_outdir[MAX_PATH] = "";
int          width = 130,
             height = 130,
             moving = 0,
             frame = 0,
             encoding_done = 0,
             encoding,
             stop_decoding,
             stop_encoding,
             totalfiles,
             numfiles,
             animate = 0;
double       file_complete;
HANDLE       event = NULL;
RECT         bar1,
             bar2,
             vbrBR;
POINT        pt;
HINSTANCE    hinst;
HBITMAP      hbm[12],
             temp;
HMENU        menu;
HWND         g_hwnd,
             qcwnd;
HFONT        font2;
SettingsSPEEX  iniSettings;     // iniSettings holds the parameters for the speexdrop-configuration.

static const char *resampleCaption[] =
{
	"48000", "44100", "32000", "24000", "22050", "16000", "11025", "8000", 0
};

static const char *EnumCaption[] =
{
	"0", "1", "2", "3", "4", "5", "6", "7", "8", "9", "10", 0
};

typedef union
{
	int buflen;
	char buf[16];
} EBUF;

LRESULT CALLBACK WndProc(HWND hwnd, UINT message, WPARAM wParam, LPARAM lParam);

BOOL CALLBACK EncodeProc(HWND hwndDlg, UINT message, WPARAM wParam, LPARAM lParam) ;
BOOL CALLBACK DecodeProc(HWND hwndDlg, UINT message, WPARAM wParam, LPARAM lParam) ;
BOOL CALLBACK DirProc(HWND hwndDlg, UINT message, WPARAM wParam, LPARAM lParam) ;
BOOL CALLBACK AboutProc(HWND hwndDlg, UINT message, WPARAM wParam, LPARAM lParam) ;

// Next function for Output Directory options

static int CALLBACK BrowseCallbackProc( HWND hwnd, UINT uMsg, LPARAM lParam, LPARAM lpData)
{
	if (uMsg == BFFM_INITIALIZED)
	{
		SetWindowText(hwnd,"Select Output Directory");
		SendMessage(hwnd,BFFM_SETSELECTION,(WPARAM)1,(LPARAM)config_outdir);
	}
	return 0;
}

/*
 * Write the .ini file using the current speexdrop settings
 */
int
WriteIniFile ( const char* Filename )
{
	FILE* fp;

	if ( (fp = fopen (Filename, "w")) == NULL )
		return EOF;                             // could not open file

	fprintf (fp, "[SpeexDrop]\n");
	fprintf (fp, "Window_X=%i\n"          , iniSettings.window_x     );
	fprintf (fp, "Window_Y=%i\n"          , iniSettings.window_y     );
	fprintf (fp, "Always_on_top=%i\n"     , iniSettings.always_on_top);
	fprintf (fp, "Logerr=%i\n"            , iniSettings.logerr       );
	fprintf (fp, "Quality=%i\n"           , iniSettings.quality      );
	fprintf (fp, "EncoderComplexity=%i\n" , iniSettings.complexity   );
	fprintf (fp, "NumberOfFrames=%i\n"    , iniSettings.frames       );
	fprintf (fp, "MaxBitrate=%i\n"        , iniSettings.max_bitrate  );
	fprintf (fp, "ABRBitrate=%i\n"        , iniSettings.abr_bitrate  );
	fprintf (fp, "EnableVBR=%i\n"         , iniSettings.vbr          );
	fprintf (fp, "EnableVAD=%i\n"         , iniSettings.vad          );
	fprintf (fp, "EnableDTX=%i\n"         , iniSettings.dtx          );
	fprintf (fp, "DeleteInput=%i\n"       , iniSettings.delete_input );
	fprintf (fp, "Volume=%i\n"            , iniSettings.volume       );
	fprintf (fp, "Limiter=%i\n"           , iniSettings.limiter      );
	fprintf (fp, "Dither=%i\n"            , iniSettings.dither       );
	fprintf (fp, "DecodeMode=%i\n"        , iniSettings.decode_mode  );
	fprintf (fp, "WaveFormat=%i\n"        , iniSettings.wav_format   );
	fprintf (fp, "ReSampleRate=%i\n"      , iniSettings.resample_rate);
	fprintf (fp, "ReSampleMode=%i\n"      , iniSettings.resample_mode);
	fprintf (fp, "Downmix=%i\n"           , iniSettings.downmix);
	fprintf (fp, "OutputDirMode=%i\n"     , iniSettings.out_dir_mode );
	fprintf (fp, "OutputDir=%s\n"         , iniSettings.out_dir      );
	return fclose (fp);
}

/*
 * Read the .ini file and set the speexdrop settings
 */
int
ReadIniFile ( FILE* fp )
{
	char  buff [256];
	int   val;

	rewind ( fp );
	fgets ( buff, sizeof buff, fp );

	if ( 0 != memcmp ( buff, "[SpeexDrop]", 11 ) )
		return EOF;

	while ( fgets ( buff, sizeof buff, fp ) != NULL ) {
		if      ( 1 == sscanf ( buff, "Window_X=%d"         , &val ) )  iniSettings.window_x      = val;
		else if ( 1 == sscanf ( buff, "Window_Y=%d"         , &val ) )  iniSettings.window_y      = val;
		else if ( 1 == sscanf ( buff, "Always_on_top=%d"    , &val ) )  iniSettings.always_on_top = val;
		else if ( 1 == sscanf ( buff, "Logerr=%d"           , &val ) )  iniSettings.logerr        = val;
		else if ( 1 == sscanf ( buff, "Quality=%d"          , &val ) )  iniSettings.quality       = val;
		else if ( 1 == sscanf ( buff, "EncoderComplexity=%d", &val ) )  iniSettings.complexity    = val;
		else if ( 1 == sscanf ( buff, "NumberOfFrames=%d"   , &val ) )  iniSettings.frames        = val;
		else if ( 1 == sscanf ( buff, "MaxBitrate=%d"       , &val ) )  iniSettings.max_bitrate   = val;
		else if ( 1 == sscanf ( buff, "ABRBitrate=%d"       , &val ) )  iniSettings.abr_bitrate   = val;
		else if ( 1 == sscanf ( buff, "EnableVBR=%d"        , &val ) )  iniSettings.vbr           = val;
		else if ( 1 == sscanf ( buff, "EnableVAD=%d"        , &val ) )  iniSettings.vad           = val;
		else if ( 1 == sscanf ( buff, "EnableDTX=%d"        , &val ) )  iniSettings.dtx           = val;
		else if ( 1 == sscanf ( buff, "DeleteInput=%d"      , &val ) )  iniSettings.delete_input  = val;
		else if ( 1 == sscanf ( buff, "Volume=%d"           , &val ) )  iniSettings.volume        = val;
		else if ( 1 == sscanf ( buff, "Limiter=%d"          , &val ) )  iniSettings.limiter       = val;
		else if ( 1 == sscanf ( buff, "Dither=%d"           , &val ) )  iniSettings.dither        = val;
		else if ( 1 == sscanf ( buff, "DecodeMode=%d"       , &val ) )  iniSettings.decode_mode   = val;
		else if ( 1 == sscanf ( buff, "WaveFormat=%d"       , &val ) )  iniSettings.wav_format    = val;
		else if ( 1 == sscanf ( buff, "ReSampleRate=%d"     , &val ) )  iniSettings.resample_rate = val;
		else if ( 1 == sscanf ( buff, "ReSampleMode=%d"     , &val ) )  iniSettings.resample_mode = val;
		else if ( 1 == sscanf ( buff, "Downmix=%d"          , &val ) )  iniSettings.downmix       = val;
		else if ( 1 == sscanf ( buff, "OutputDirMode=%d"    , &val ) )  iniSettings.out_dir_mode  = val;
		else if ( 1 == sscanf ( buff, "OutputDir=%245[^\r\n]"     , iniSettings.out_dir ) );
	}

	return 0;
}

/*
 * Get speexdrop settings at startup, writes .ini file, if not present
 */
void
GetSpeexdropSettings ( void )
{
	FILE*  fp = NULL;
	char   PathAndName [] = {INI_FILE};

	// set default values
	iniSettings.window_x      =   64;        // default box position (x co-ord)
	iniSettings.window_y      =   64;        // default box position (y co-ord)
	iniSettings.always_on_top =    8;        // default = on
	iniSettings.logerr        =    0;        // default = off
	iniSettings.quality       =    3;        // default quality = 3
	iniSettings.complexity    =    3;        // default = 3
	iniSettings.frames        =    1;        // default = 1
	iniSettings.max_bitrate   =    0;        // default = 0
	iniSettings.abr_bitrate   =    0;        // default = 0
	iniSettings.vbr           =    0;        // default = off
	iniSettings.vad           =    0;        // default = off
	iniSettings.dtx           =    0;        // default = off
	iniSettings.delete_input  =    0;        // default = off
	iniSettings.volume        =    0;        // default = 0dB
	iniSettings.limiter       =    0;        // default = off
	iniSettings.dither        =    0;        // default = don't dither output
	iniSettings.decode_mode   =    1;        // default = 1 (decode to file)
	iniSettings.wav_format    =    1;        // default = 1 (16 bit PCM)
	iniSettings.resample_rate =    0;        // default = No resampling
	iniSettings.resample_mode =    0;        // default = No resampling
	iniSettings.downmix       =    1;        // default = 1 (downmix)
	iniSettings.out_dir_mode  =    0;        // default = same as input dir
	strcpy(iniSettings.out_dir, "");         // default = blank (as above)

	// Read INI_FILE
	if ( (fp = fopen (PathAndName, "r")) == NULL ) {    // file does not exist: write it!
		WriteIniFile ( PathAndName );
	}
	else {                                              // file does exist: read it!
		ReadIniFile (fp);
		fclose (fp);
	}

	return;
}

void set_quality_coefficient(int v)
{
	encthread_set_quality_coefficient(v);
	iniSettings.quality = v;
}

void set_enc_complexity(int v)
{
	encthread_set_enc_complexity(v);
	iniSettings.complexity = v;
}

void set_no_of_frames(int v)
{
	encthread_set_no_of_frames(v);
	iniSettings.frames = v;
}

void set_max_bitrate(int v)
{
	encthread_set_max_bitrate(v);
	iniSettings.max_bitrate = v;
}

void set_abr_bitrate(int v)
{
	encthread_set_abr_bitrate(v);
	iniSettings.abr_bitrate = v;
}

void set_enable_vbr(int v)
{
	encthread_set_enable_vbr(v);
	iniSettings.vbr = v;
}

void set_enable_vad(int v)
{
	encthread_set_enable_vad(v);
	iniSettings.vad = v;
}

void set_enable_dtx(int v)
{
	encthread_set_enable_dtx(v);
	iniSettings.dtx = v;
}

void set_out_dir_mode(int v) {
	iniSettings.out_dir_mode = v;
}

void set_out_dir(char v[MAX_PATH]) {
	strcpy(iniSettings.out_dir, v);
}

void set_always_on_top(HWND hwnd, int v)
{
	CheckMenuItem(menu, IDM_ONTOP, v ? MF_CHECKED : MF_UNCHECKED);
	SetWindowPos(hwnd, v ? HWND_TOPMOST : HWND_NOTOPMOST, 0, 0, 0, 0, SWP_NOSIZE | SWP_SHOWWINDOW | SWP_NOMOVE);
	iniSettings.always_on_top = v;
}

void set_logerr(HWND hwnd, int v)
{
	CheckMenuItem(menu, IDM_LOGERR, v ? MF_CHECKED : MF_UNCHECKED);
	iniSettings.logerr = v;
	set_use_dialogs(v);
	encthread_set_logerr(v);
}

void set_delInput(HWND hwnd, int v)
{
	CheckMenuItem(menu, IDM_DELINPUT, v ? MF_CHECKED : MF_UNCHECKED);
	encthread_set_delInput(v);
	iniSettings.delete_input = v;
}

void set_downmix(int v)
{
	encthread_set_downmix(v);
	iniSettings.downmix = v;
}

void set_resample_rate(int v)
{
	encthread_set_resample_rate(v);
	if(v != 0)
		iniSettings.resample_rate = v;
}

void set_resample_mode(int v)
{
	iniSettings.resample_mode = v;
}

void set_volume(int v)
{
	decthread_set_volume(v);
	iniSettings.volume = v;
}

void set_limiter(int v)
{
	decthread_set_limiter(v);
	iniSettings.limiter = v;
}

void set_dither(int v)
{
	decthread_set_dither(v);
	iniSettings.dither = v;
}

void set_decode_mode(int v)
{
	decthread_set_decode_mode(v);
	iniSettings.decode_mode = v;
}

void set_wav_format(int v)
{
	decthread_set_wav_format(v);
	iniSettings.wav_format = v;
}

int WINAPI WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance, PSTR szCmdLine, int iCmdShow)
{
	static char szAppName[] = "speexdropWin";
	HWND        hwnd;
	MSG         msg;
	WNDCLASS    wndclass;
	const int   width = 130,
	            height = 130;
	int         x,
	            y;

	hinst = hInstance;

	wndclass.style = CS_HREDRAW | CS_VREDRAW;
	wndclass.lpfnWndProc = WndProc;
	wndclass.cbClsExtra = 0;
	wndclass.cbWndExtra = 0;
	wndclass.hInstance = hInstance;
	wndclass.hIcon = LoadIcon(hinst, MAKEINTRESOURCE(IDI_ICON1));
	wndclass.hCursor = LoadCursor(NULL, IDC_ARROW);
	wndclass.hbrBackground = (HBRUSH)GetStockObject(WHITE_BRUSH);
	wndclass.lpszMenuName = NULL;
	wndclass.lpszClassName = szAppName;

	RegisterClass(&wndclass);

	GetSpeexdropSettings();

	x = max(min(iniSettings.window_x, GetSystemMetrics(SM_CXSCREEN) - width), 0);
	y = max(min(iniSettings.window_y, GetSystemMetrics(SM_CYSCREEN) - height), 0);

	hwnd = CreateWindow(szAppName, "speexdrop", WS_POPUP | WS_DLGFRAME, x, y,
	width, height, NULL, NULL, hInstance, NULL);

	g_hwnd = hwnd;

	ShowWindow(hwnd, iCmdShow);
	UpdateWindow(hwnd);

	font2 = CREATEFONT(10);

	SetTimer(hwnd, 1, 80, NULL);

	set_always_on_top(hwnd, iniSettings.always_on_top);
	set_logerr(hwnd, iniSettings.logerr);
	set_delInput(hwnd, iniSettings.delete_input);
	set_quality_coefficient(iniSettings.quality);
	set_enc_complexity(iniSettings.complexity);
	set_no_of_frames(iniSettings.frames);
	set_max_bitrate(iniSettings.max_bitrate);
	set_abr_bitrate(iniSettings.abr_bitrate);
	set_enable_vbr(iniSettings.vbr);
	set_enable_vad(iniSettings.vad);
	set_enable_dtx(iniSettings.dtx);
	set_volume(iniSettings.volume);
	set_limiter(iniSettings.limiter);
	set_dither(iniSettings.dither);
	set_decode_mode(iniSettings.decode_mode);
	set_wav_format(iniSettings.wav_format);
	set_downmix(iniSettings.downmix);
	set_resample_mode(iniSettings.resample_mode);
	set_resample_rate(!iniSettings.resample_mode ? 0 : iniSettings.resample_rate);
	if(iniSettings.out_dir_mode) {
		set_out_dir(iniSettings.out_dir);
		strcpy(config_outdir, iniSettings.out_dir);
	}
	else {
		set_out_dir("");
		strcpy(config_outdir, "");
	}

	for (frame = 0; frame < 10; frame++)
		hbm[frame] = LoadImage(hinst, MAKEINTRESOURCE(IDB_TF01 + frame), IMAGE_BITMAP, 0, 0, LR_CREATEDIBSECTION);
	frame = 0;

	while (GetMessage(&msg, NULL, 0, 0)) {
		TranslateMessage(&msg);
		DispatchMessage(&msg);
	}

	for (frame = 0; frame < 10; frame++) 
		DeleteObject(hbm[frame]);

	return msg.wParam;
}

void HandleDrag(HWND hwnd, HDROP hDrop)
{
	int    cFiles,
	       i,
	       flag = 0;
	char   szFile[MAX_PATH],
	       *ext;

	cFiles = DragQueryFile(hDrop, 0xFFFFFFFF, NULL, 0);
	for (i = 0; i < cFiles; i++) {
		DragQueryFile(hDrop, i, szFile, sizeof(szFile));

		if (ext = strrchr(szFile, '.'))	{
			if ((stricmp(ext, ".wav") == 0) || (stricmp(ext, ".aif") == 0))	{
				flag = 1;
				encthread_addfile(szFile);
				encoding = 1;
				stop_encoding = 0;
			}
			else if (stricmp(ext, ".spx") == 0){
				flag = 1;
				encthread_addfile(szFile);
				encoding = 0;
				stop_decoding = 0;
			}
			else
				MessageBox(NULL, "Unsupported input file type", "Error", 0);
		}
	}

	DragFinish(hDrop);

	if (flag)
		SetEvent(event);
}

LRESULT CALLBACK WndProc(HWND hwnd, UINT message, WPARAM wParam, LPARAM lParam)
{
	HDC          hdc,
	             hmem;
	static HDC   offscreen;
	PAINTSTRUCT  ps;
	RECT         rect;
	BITMAP       bm;
	POINT        point;
	static POINT start;
	static int   dragging = 0;
	HDC          desktop;
	HBITMAP      hbitmap;
	HANDLE       hdrop;
	HFONT        dfltFont;
	int          dfltBGMode;
	double       percomp;

	switch (message) {
		case WM_CREATE:
			menu = LoadMenu(hinst, MAKEINTRESOURCE(IDR_MENU1));
			menu = GetSubMenu(menu, 0);

			offscreen = CreateCompatibleDC(NULL);
			desktop = GetDC(GetDesktopWindow());
			hbitmap = CreateCompatibleBitmap(desktop, 200, 200);
			ReleaseDC(GetDesktopWindow(), desktop);
			SelectObject(offscreen, hbitmap);

			// Start the engines
			encthread_init();

			// We accept drag&drop
			DragAcceptFiles(hwnd, TRUE);
			return 0;

		case WM_PAINT:
			hdc = BeginPaint(hwnd, &ps);
			GetClientRect(hwnd, &rect);
			width = rect.right + 1;
			height = rect.bottom + 1;

			FillRect(offscreen, &rect, (HBRUSH)GetStockObject(WHITE_BRUSH));
			DrawText(offscreen, "Drop Files Here", -1, &rect, DT_SINGLELINE | DT_CENTER);

			hmem = CreateCompatibleDC(offscreen);
			SelectObject(hmem, hbm[frame]);
			GetObject(hbm[frame], sizeof(BITMAP), &bm);
			BitBlt(offscreen, width / 2 - 50, height / 2 - 33, bm.bmWidth, bm.bmHeight, hmem, 0, 0, SRCCOPY);
			DeleteDC(hmem);
		
			percomp = ((double)(totalfiles - numfiles) + 1 - (1 - file_complete)) / (double)totalfiles;

			SetRect(&vbrBR, 0, height - 35, width, height - 19);

			dfltBGMode = SetBkMode(offscreen, TRANSPARENT);
			dfltFont = SelectObject(offscreen, font2);


			SetRect(&bar1, 0, height - 23, (int)(file_complete * width), height - 13);
			SetRect(&bar2, 0, height - 12, (int)(percomp * width), height - 2);

			FillRect(offscreen, &bar1, (HBRUSH)GetStockObject(LTGRAY_BRUSH));
			FillRect(offscreen, &bar2, (HBRUSH)GetStockObject(DKGRAY_BRUSH));

			if (fileName){
				char* sep;
				char  fileCaption[80];

				if ((sep = strrchr(fileName, '\\')) != 0)
					fileName = sep+1;

				(void) strcpy(fileCaption, "   ");
				(void) strcat(fileCaption, fileName);

				DrawText(offscreen, fileCaption, -1, &bar1, DT_SINGLELINE | DT_LEFT);
			}

			SelectObject(offscreen, dfltFont);
			SetBkMode(offscreen, dfltBGMode);

			BitBlt(hdc, 0, 0, width, height, offscreen, 0, 0, SRCCOPY);

			EndPaint(hwnd, &ps);

			return DefWindowProc(hwnd, message, wParam, lParam);
			//return 0;

		case WM_TIMER:
			if (animate || frame) {
				frame--;
				if (frame < 0) 
					frame += 10;
			}
			else {
				frame = 0;
			}
			GetClientRect(hwnd, &rect);
			InvalidateRect(hwnd, &rect, FALSE);
			return 0;

		case WM_LBUTTONDOWN:
			start.x = LOWORD(lParam);
			start.y = HIWORD(lParam);
			ClientToScreen(hwnd, &start);
			GetWindowRect(hwnd, &rect);
			start.x -= rect.left;
			start.y -= rect.top;
			dragging = 1;
			SetCapture(hwnd);
			return 0;

		case WM_LBUTTONUP:
			if (dragging) {
				dragging = 0;
				ReleaseCapture();
			}
			return 0;

		case WM_MOUSEMOVE:
			if (dragging) {
				point.x = LOSHORT(lParam);
				point.y = HISHORT(lParam);

				/* lParam can contain negative coordinates !
				point.x = LOWORD(lParam);
				point.y = HIWORD(lParam);
				*/

				ClientToScreen(hwnd, &point);
				SetWindowPos(hwnd, 0, point.x - start.x, point.y - start.y, 0, 0, SWP_NOSIZE | SWP_NOZORDER | SWP_SHOWWINDOW);
				iniSettings.window_x = point.x - start.x;
				iniSettings.window_y = point.y - start.y;
			}
			return 0;

		case WM_CAPTURECHANGED:
			if (dragging) {
				dragging = 0;
				ReleaseCapture();
			}
			return 0;

		case WM_RBUTTONUP:
			point.x = LOWORD(lParam);
			point.y = HIWORD(lParam);
			ClientToScreen(hwnd, &point);
			TrackPopupMenu(menu, TPM_RIGHTBUTTON, point.x, point.y, 0, hwnd, NULL);
			return 0;

		case WM_COMMAND:
			switch (LOWORD(wParam)){
 
				case IDM_QUIT:
					WriteIniFile(INI_FILE);
					encoding_done = 1;
					PostQuitMessage(0);
					break;
				case IDM_ONTOP:
					set_always_on_top(hwnd, ~GetMenuState(menu, LOWORD(wParam), MF_BYCOMMAND) & MF_CHECKED);
					break;	
				case IDM_LOGERR:
					set_logerr(hwnd, ~GetMenuState(menu, LOWORD(wParam), MF_BYCOMMAND) & MF_CHECKED);
					break;
				case IDM_DELINPUT:
					set_delInput(hwnd, ~GetMenuState(menu, LOWORD(wParam), MF_BYCOMMAND) & MF_CHECKED);
					break;
				case IDM_STOP_ENC:{
					int v = ~GetMenuState(menu, LOWORD(wParam), MF_BYCOMMAND) & MF_CHECKED;
					if(v == 8)
						stop_encoding = 1;
					break;
				}
				case IDM_STOP_DEC:{
					int v = ~GetMenuState(menu, LOWORD(wParam), MF_BYCOMMAND) & MF_CHECKED;
					if(v == 8)
						stop_decoding = 1;
					break;
				}
				case IDM_SAVEQUALITY:{
					DialogBox(hinst, MAKEINTRESOURCE(IDD_QUALITY), hwnd, EncodeProc);

					break;
				}

				case IDM_VOLUME:{
					DialogBox(hinst, MAKEINTRESOURCE(IDD_VOLUME), hwnd, DecodeProc);

					break;
				}

				case IDM_SAVEDIR:{
					DialogBox(hinst, MAKEINTRESOURCE(IDD_DIRECTORY), hwnd, DirProc);

					break;
				}

				case IDM_ABOUT:{
					DialogBox(hinst, MAKEINTRESOURCE(IDD_ABOUT), hwnd, AboutProc);

					break;
				}
			} // LOWORD(wParam)
			return 0;

		case WM_DROPFILES:
			hdrop = (HANDLE)wParam;
			HandleDrag(hwnd, hdrop);
			return 0;
		

		case WM_DESTROY:
			encoding_done = 1;
			PostQuitMessage(0);
			return 0;
	}

		return DefWindowProc(hwnd, message, wParam, lParam);
}

int sliding=FALSE;

/*
 *  Encode parameters dialog procedures.
 */
BOOL CALLBACK EncodeProc(HWND hwndDlg, UINT message, WPARAM wParam, LPARAM lParam) 
{
	char     editBuf[14];
	int      br;

	switch (message){ 
		case WM_INITDIALOG: 
 
			for (br=0; EnumCaption[br] != 0; br++){
				SendDlgItemMessage(hwndDlg, IDC_QUALITY, CB_ADDSTRING,
	        	  		(WPARAM)0, (LPARAM) EnumCaption[br]);

				SendDlgItemMessage(hwndDlg, IDC_QUALITY, CB_SETITEMDATA, 
	        	  		(WPARAM) br, (LPARAM) atoi(EnumCaption[br]));

				SendDlgItemMessage(hwndDlg, IDC_ENC_COMPLEXITY, CB_ADDSTRING,
        	  			(WPARAM)0, (LPARAM) EnumCaption[br]);

				SendDlgItemMessage(hwndDlg, IDC_ENC_COMPLEXITY, CB_SETITEMDATA, 
        			  	(WPARAM) br, (LPARAM) atoi(EnumCaption[br]));

				SendDlgItemMessage(hwndDlg, IDC_FRAMES, CB_ADDSTRING,
        			  	(WPARAM)0, (LPARAM) EnumCaption[br]);

				SendDlgItemMessage(hwndDlg, IDC_FRAMES, CB_SETITEMDATA, 
        			  	(WPARAM) br, (LPARAM) atoi(EnumCaption[br]));
			}

			if(iniSettings.vbr)
				CheckDlgButton(hwndDlg,IDC_USE_VBR,TRUE);
			else if(iniSettings.vad) {
				CheckDlgButton(hwndDlg,IDC_VAD,TRUE);
				if(iniSettings.dtx)
					CheckDlgButton(hwndDlg,IDC_VAD,TRUE);
			}

			for (br=0; resampleCaption[br] != 0; br++){
				SendDlgItemMessage(hwndDlg, IDC_RESAMPLE_RATE, CB_ADDSTRING,
	        	  		(WPARAM)0, (LPARAM) resampleCaption[br]);

				SendDlgItemMessage(hwndDlg, IDC_RESAMPLE_RATE, CB_SETITEMDATA, 
	        	  		(WPARAM) br, (LPARAM) atoi(resampleCaption[br]));
			}

			/* set to saved sample rate */
			for (br=0; resampleCaption[br] != 0; br++){
				if ( atoi(resampleCaption[br]) == iniSettings.resample_rate){
					SendDlgItemMessage(hwndDlg, IDC_RESAMPLE_RATE, CB_SETCURSEL,
	        	  			(WPARAM) br, (LPARAM) 0);
					break;
				}
			}

			if(iniSettings.resample_mode == 0){
				EnableWindow(GetDlgItem(hwndDlg, IDC_RESAMPLE_RATE), FALSE);
				EnableWindow(GetDlgItem(hwndDlg, IDC_RESAMPLE_THIS), FALSE);
			}
			else if(iniSettings.resample_mode == 1){
				CheckDlgButton(hwndDlg, IDC_RESAMPLE_NEW_DEFAULT, TRUE);
				EnableWindow(GetDlgItem(hwndDlg, IDC_RESAMPLE_RATE), TRUE);
				EnableWindow(GetDlgItem(hwndDlg, IDC_RESAMPLE_THIS), TRUE);
			}

			if(iniSettings.downmix){
				CheckDlgButton(hwndDlg,IDC_DOWNMIX,TRUE);
			}

		   	(void) sprintf(editBuf, "%d", iniSettings.quality);

		   	for(br=0; EnumCaption[br] != 0; br++){
				if ( ! strcmp(EnumCaption[br], editBuf) ){
        				SendDlgItemMessage(hwndDlg, IDC_QUALITY, CB_SETCURSEL,
			            		(WPARAM) br, (LPARAM) 0);
        				break;
				}
			}

		   	(void) sprintf(editBuf, "%d", iniSettings.complexity);

		   	for(br=0; EnumCaption[br] != 0; br++){
				if ( ! strcmp(EnumCaption[br], editBuf) ){
        				SendDlgItemMessage(hwndDlg, IDC_ENC_COMPLEXITY, CB_SETCURSEL,
			            		(WPARAM) br, (LPARAM) 0);
        				break;
				}
			}

			(void) sprintf(editBuf, "%d", iniSettings.frames);

			for(br=0; EnumCaption[br] != 0; br++){
				if ( ! strcmp(EnumCaption[br], editBuf) ){
					SendDlgItemMessage(hwndDlg, IDC_FRAMES, CB_SETCURSEL,
            					(WPARAM) br, (LPARAM) 0);
					break;
				}
			}

			sprintf(editBuf, "%d", iniSettings.max_bitrate);

			SendDlgItemMessage(hwndDlg, IDC_MAX_BITRATE, WM_SETTEXT,
				(WPARAM)0, (LPARAM)editBuf);

			sprintf(editBuf, "%d", iniSettings.abr_bitrate);

			SendDlgItemMessage(hwndDlg, IDC_ABR_BITRATE, WM_SETTEXT,
				(WPARAM)0, (LPARAM)editBuf);

   			break;

		case WM_CLOSE:
			EndDialog(hwndDlg, TRUE);
			return TRUE;

		case WM_COMMAND: 
			switch (LOWORD(wParam)) {
				case IDC_BUTTON1: {
					char szTemp[12];
					if (IsDlgButtonChecked(hwndDlg, IDC_USE_VBR) == BST_CHECKED)
						set_enable_vbr(1);
					else
						set_enable_vbr(0);

					if (IsDlgButtonChecked(hwndDlg, IDC_VAD) == BST_CHECKED)
						set_enable_vad(1);
					else
						set_enable_vad(0);

					if (IsDlgButtonChecked(hwndDlg, IDC_DTX) == BST_CHECKED)
						set_enable_dtx(1);
					else
						set_enable_dtx(0);

					if (IsDlgButtonChecked(hwndDlg, IDC_DOWNMIX) == BST_CHECKED)
						set_downmix(1);               // Downmix to mono
					else
						set_downmix(0);

					if (IsDlgButtonChecked(hwndDlg, IDC_RESAMPLE_NEW_DEFAULT) == BST_CHECKED){
						set_resample_mode(1);
						if (IsDlgButtonChecked(hwndDlg, IDC_RESAMPLE_THIS) == BST_CHECKED)
								set_resample_mode(0);
						if ((br = SendDlgItemMessage(hwndDlg, IDC_RESAMPLE_RATE, CB_GETCURSEL,
								(WPARAM) 0, (LPARAM) 0)) != CB_ERR){
							set_resample_rate(SendDlgItemMessage(hwndDlg, IDC_RESAMPLE_RATE,
									CB_GETITEMDATA, (WPARAM) br, (LPARAM) 0));
						}
					}
					else {
						set_resample_mode(0);
						set_resample_rate(0);
					}

					if ((br = SendDlgItemMessage(hwndDlg, IDC_QUALITY, CB_GETCURSEL,
							(WPARAM) 0, (LPARAM) 0)) != CB_ERR){
						set_quality_coefficient(SendDlgItemMessage(hwndDlg, IDC_QUALITY, 
								CB_GETITEMDATA,	(WPARAM) br, (LPARAM) 0));
					}

					if ((br = SendDlgItemMessage(hwndDlg, IDC_ENC_COMPLEXITY, CB_GETCURSEL,
							(WPARAM) 0, (LPARAM) 0)) != CB_ERR){
						set_enc_complexity(SendDlgItemMessage(hwndDlg, IDC_ENC_COMPLEXITY, 
								CB_GETITEMDATA,	(WPARAM) br, (LPARAM) 0));
					}

					if ((br = SendDlgItemMessage(hwndDlg, IDC_FRAMES, CB_GETCURSEL,
							(WPARAM) 0, (LPARAM) 0)) != CB_ERR){
						set_no_of_frames(SendDlgItemMessage(hwndDlg, IDC_FRAMES, 
								CB_GETITEMDATA,	(WPARAM) br, (LPARAM) 0));
					}

					GetDlgItemText(hwndDlg, IDC_MAX_BITRATE, szTemp, sizeof(szTemp));
					if(strlen(szTemp) > 0)
						set_max_bitrate(atoi(szTemp));
					else
						set_max_bitrate(0);

					GetDlgItemText(hwndDlg, IDC_ABR_BITRATE, szTemp, sizeof(szTemp));
					if(strlen(szTemp) > 0)
						set_abr_bitrate(atoi(szTemp));
					else
						set_abr_bitrate(0);

					EndDialog(hwndDlg, TRUE);
					return TRUE;
				}

				case IDC_USE_VBR:
					if (IsDlgButtonChecked(hwndDlg, IDC_USE_VBR) == BST_CHECKED){
						CheckDlgButton(hwndDlg,IDC_VAD,FALSE);
						CheckDlgButton(hwndDlg,IDC_DTX,FALSE);
					}
					break;

				case IDC_VAD:
					if (IsDlgButtonChecked(hwndDlg, IDC_VAD) == BST_CHECKED)
						CheckDlgButton(hwndDlg,IDC_USE_VBR,FALSE);

					if ((IsDlgButtonChecked(hwndDlg, IDC_VAD) != BST_CHECKED)
					&& (IsDlgButtonChecked(hwndDlg, IDC_DTX) == BST_CHECKED)) {
						CheckDlgButton(hwndDlg,IDC_VAD,TRUE);
						CheckDlgButton(hwndDlg,IDC_USE_VBR,FALSE);
					}
					break;

				case IDC_DTX:
					if (IsDlgButtonChecked(hwndDlg, IDC_DTX) == BST_CHECKED){
						CheckDlgButton(hwndDlg,IDC_VAD,TRUE);
						CheckDlgButton(hwndDlg,IDC_USE_VBR,FALSE);
					}
					break;

				case IDC_RESAMPLE_NEW_DEFAULT:
					if (IsDlgButtonChecked(hwndDlg, IDC_RESAMPLE_NEW_DEFAULT) == BST_CHECKED) {
						EnableWindow(GetDlgItem(hwndDlg, IDC_RESAMPLE_RATE), TRUE);
						EnableWindow(GetDlgItem(hwndDlg, IDC_RESAMPLE_THIS), TRUE);
					}
					else {
						EnableWindow(GetDlgItem(hwndDlg, IDC_RESAMPLE_RATE), FALSE);
						EnableWindow(GetDlgItem(hwndDlg, IDC_RESAMPLE_THIS), FALSE);
					}
					break;

        	  	}
	}
	return FALSE; 
}

/*
 *  Decode parameters dialog procedures.
 */
BOOL CALLBACK DecodeProc(HWND hwndDlg, UINT message, WPARAM wParam, LPARAM lParam) 
{
	int      Limiter,
	         DecodeMode,
	         WavFormat;

	switch (message){ 
		case WM_INITDIALOG: 
 
			SendDlgItemMessage(hwndDlg, IDC_SLIDER2, TBM_SETRANGE, 
        			(WPARAM) TRUE,                   // redraw flag 
        			(LPARAM) MAKELONG(-12, 12));  // min. & max. positions 

			SendDlgItemMessage(hwndDlg, IDC_SLIDER2, TBM_SETPAGESIZE, 
        			0, (LPARAM) 1);                  // new page size 
 
		   	SendDlgItemMessage(hwndDlg, IDC_SLIDER2, TBM_SETSEL, 
        			(WPARAM) FALSE,                  // redraw flag 
		        	(LPARAM) MAKELONG(-12, 12));

		   	SendDlgItemMessage(hwndDlg, IDC_SLIDER2, TBM_SETPOS, 
        			(WPARAM) TRUE,                   // redraw flag 
		        	(LPARAM) iniSettings.volume);

			if(iniSettings.decode_mode == 0){
				DecodeMode = 1038;
				CheckDlgButton(hwndDlg,IDC_16BIT,TRUE);
				CheckDlgButton(hwndDlg,IDC_8BIT,FALSE);
				CheckDlgButton(hwndDlg,IDC_24BIT,FALSE);
				CheckDlgButton(hwndDlg,IDC_32BIT,FALSE);
				CheckDlgButton(hwndDlg,IDC_FLOATS,FALSE);
				EnableWindow(GetDlgItem(hwndDlg, IDC_8BIT), FALSE);
				EnableWindow(GetDlgItem(hwndDlg, IDC_24BIT), FALSE);
				EnableWindow(GetDlgItem(hwndDlg, IDC_32BIT), FALSE);
				EnableWindow(GetDlgItem(hwndDlg, IDC_FLOATS), FALSE);
			}
			else if(iniSettings.decode_mode == 1){
				DecodeMode = 1039;
				EnableWindow(GetDlgItem(hwndDlg, IDC_8BIT), TRUE);
				EnableWindow(GetDlgItem(hwndDlg, IDC_24BIT), TRUE);
				EnableWindow(GetDlgItem(hwndDlg, IDC_32BIT), TRUE);
				EnableWindow(GetDlgItem(hwndDlg, IDC_FLOATS), TRUE);
			}
			(void) CheckRadioButton(hwndDlg, IDC_PLAYBACK,
											 IDC_DECODE,
											 DecodeMode);

			if(iniSettings.limiter == 0)
				CheckDlgButton(hwndDlg,IDC_HARD_LIMITER,FALSE);
			else if(iniSettings.limiter == 1)
				CheckDlgButton(hwndDlg,IDC_HARD_LIMITER,TRUE);

			if(iniSettings.dither == 0)
				CheckDlgButton(hwndDlg,IDC_DITHER_OFF,TRUE);
			else if(iniSettings.dither == 1)
				CheckDlgButton(hwndDlg,IDC_DITHER,TRUE);
			else if(iniSettings.dither == 2)
				CheckDlgButton(hwndDlg,IDC_DITHER_L_SHAPE,TRUE);
			else if(iniSettings.dither == 3)
				CheckDlgButton(hwndDlg,IDC_DITHER_M_SHAPE,TRUE);
			else if(iniSettings.dither == 4)
				CheckDlgButton(hwndDlg,IDC_DITHER_H_SHAPE,TRUE);

			if(iniSettings.wav_format == 1)
				WavFormat = 1040;
			else if(iniSettings.wav_format == 2)
				WavFormat = 1041;
			else if(iniSettings.wav_format == 3)
				WavFormat = 1042;
			else if(iniSettings.wav_format == 4)
				WavFormat = 1043;
			else if(iniSettings.wav_format == 5)
				WavFormat = 1044;
			(void) CheckRadioButton(hwndDlg, IDC_8BIT,
											 IDC_FLOATS,
											 WavFormat);
   			break;

		case WM_CLOSE:
			EndDialog(hwndDlg, TRUE);
			return TRUE;

		case WM_COMMAND: 
			switch (LOWORD(wParam)) {
				case IDC_BUTTON2:{
					if (IsDlgButtonChecked(hwndDlg, IDC_HARD_LIMITER) == BST_CHECKED)
						set_limiter(1);            // Use hard limiter
					else
						set_limiter(0);            // Hard limiter off

					if (IsDlgButtonChecked(hwndDlg, IDC_DITHER) == BST_CHECKED)
						set_dither(1);            // Dither output
					else if (IsDlgButtonChecked(hwndDlg, IDC_DITHER_L_SHAPE) == BST_CHECKED)
						set_dither(2);            // Dither Light noise shaping
					else if (IsDlgButtonChecked(hwndDlg, IDC_DITHER_M_SHAPE) == BST_CHECKED)
						set_dither(3);            // Dither Medium noise shaping
					else if (IsDlgButtonChecked(hwndDlg, IDC_DITHER_H_SHAPE) == BST_CHECKED)
						set_dither(4);            // Dither Heavy noise shaping
					else
						set_dither(0);            // Don't dither output

					if (IsDlgButtonChecked(hwndDlg, IDC_PLAYBACK) == BST_CHECKED)
						set_decode_mode(0);            // Playback
					else if (IsDlgButtonChecked(hwndDlg, IDC_DECODE) == BST_CHECKED)
						set_decode_mode(1);            // Decode to File

					if (IsDlgButtonChecked(hwndDlg, IDC_8BIT) == BST_CHECKED)
						set_wav_format(1);             // 8 bit PCM
					else if (IsDlgButtonChecked(hwndDlg, IDC_16BIT) == BST_CHECKED)
						set_wav_format(2);             // 16 bit PCM
					else if (IsDlgButtonChecked(hwndDlg, IDC_24BIT) == BST_CHECKED)
						set_wav_format(3);             // 24 bit PCM
					else if (IsDlgButtonChecked(hwndDlg, IDC_32BIT) == BST_CHECKED)
						set_wav_format(4);             // 32 bit PCM
					else if (IsDlgButtonChecked(hwndDlg, IDC_FLOATS) == BST_CHECKED)
						set_wav_format(5);             // 32 bit floats

					set_volume(SendDlgItemMessage(hwndDlg, IDC_SLIDER2, TBM_GETPOS, (WPARAM)0, (LPARAM)0));

					EndDialog(hwndDlg, TRUE);
					return TRUE;
				}

				case IDC_PLAYBACK:
					CheckDlgButton(hwndDlg,IDC_DECODE,FALSE);
					CheckDlgButton(hwndDlg,IDC_16BIT,TRUE);
					CheckDlgButton(hwndDlg,IDC_8BIT,FALSE);
					CheckDlgButton(hwndDlg,IDC_24BIT,FALSE);
					CheckDlgButton(hwndDlg,IDC_32BIT,FALSE);
					CheckDlgButton(hwndDlg,IDC_FLOATS,FALSE);
					EnableWindow(GetDlgItem(hwndDlg, IDC_8BIT), FALSE);
					EnableWindow(GetDlgItem(hwndDlg, IDC_24BIT), FALSE);
					EnableWindow(GetDlgItem(hwndDlg, IDC_32BIT), FALSE);
					EnableWindow(GetDlgItem(hwndDlg, IDC_FLOATS), FALSE);
					break;

				case IDC_DECODE:
					CheckDlgButton(hwndDlg,IDC_PLAYBACK,FALSE);
					EnableWindow(GetDlgItem(hwndDlg, IDC_8BIT), TRUE);
					EnableWindow(GetDlgItem(hwndDlg, IDC_24BIT), TRUE);
					EnableWindow(GetDlgItem(hwndDlg, IDC_32BIT), TRUE);
					EnableWindow(GetDlgItem(hwndDlg, IDC_FLOATS), TRUE);
					break;

        	  	}
	}
	return FALSE; 
}

/*
 *  Directory parameters dialog procedures.
 */
BOOL CALLBACK DirProc(HWND hwndDlg, UINT message, WPARAM wParam, LPARAM lParam) 
{

	switch (message){ 
		case WM_INITDIALOG: 
 
			if(iniSettings.out_dir_mode == 1) {
				CheckDlgButton(hwndDlg,IDC_USEOTHERDEF,TRUE);
				SetDlgItemText(hwndDlg, IDC_EDITDIR, config_outdir);
			}
			else {
				CheckDlgButton(hwndDlg,IDC_USESAMEDIR,TRUE);
				EnableWindow(GetDlgItem(hwndDlg, IDC_EDITDIR), FALSE);
				EnableWindow(GetDlgItem(hwndDlg, IDC_SETDIR), FALSE);
				SetDlgItemText(hwndDlg, IDC_EDITDIR, "");
			}
			return TRUE;

		case WM_CLOSE:
			EndDialog(hwndDlg, TRUE);
			return TRUE;

		case WM_COMMAND: 
			switch (LOWORD(wParam)) {
				case IDC_BUTTON4: {
					char     szTemp[MAX_PATH];
					char     szTemp_2[MAX_PATH];
					int      i = 0;
					int      j = 0;
					char     sub[MAX_PATH];

					if (IsDlgButtonChecked(hwndDlg, IDC_USESAMEDIR) == BST_CHECKED) {
						strcpy(config_outdir, "");
						set_out_dir("");
						set_out_dir_mode(0);
					}
					else {
						GetDlgItemText(hwndDlg, IDC_EDITDIR, szTemp_2, sizeof(szTemp_2));
						if(!szTemp_2[0]) {
							set_out_dir_mode(0);
							strcpy(config_outdir, "");
						}
						else {
							j = strlen(szTemp_2) - 1;
							if(szTemp_2[j] != '\\') {
								szTemp_2[j+1] = '\\';
								szTemp_2[j+2] = '\0';
							}
							for(i = 0; i < strlen(szTemp_2); i++) {
								if(szTemp_2[i] == '\\') {
									sub[i] = szTemp_2[i];
									sub[i+1] = '\0';
									CreateDirectory(sub, NULL);
								}
								else
									sub[i] = szTemp_2[i];
							}
							sub[i-1] = '\0';
							strcpy(config_outdir, sub);
						}
						set_out_dir_mode(IsDlgButtonChecked(hwndDlg, IDC_USEOTHERDEF) == BST_CHECKED ? 1 : 0);
						set_out_dir(config_outdir);
					}

					EndDialog(hwndDlg, FALSE);
				}
				break;

				case IDC_USESAMEDIR:
					SetDlgItemText(hwndDlg, IDC_EDITDIR, "");
					EnableWindow(GetDlgItem(hwndDlg, IDC_EDITDIR), FALSE);
					EnableWindow(GetDlgItem(hwndDlg, IDC_SETDIR), FALSE);
					break;
									
				case IDC_USEOTHERDIR:
					EnableWindow(GetDlgItem(hwndDlg, IDC_EDITDIR), TRUE);
					EnableWindow(GetDlgItem(hwndDlg, IDC_SETDIR), TRUE);
					break;

				case IDC_USEOTHERDEF:
					EnableWindow(GetDlgItem(hwndDlg, IDC_EDITDIR), TRUE);
					EnableWindow(GetDlgItem(hwndDlg, IDC_SETDIR), TRUE);
					break;

				case IDC_SETDIR: {
					char        szTemp[MAX_PATH];
					char        name[MAX_PATH];
					BROWSEINFO  bi;
					ITEMIDLIST  *idlist;

					bi.hwndOwner = hwndDlg;
					bi.pidlRoot = 0;
					bi.pszDisplayName = name;
					bi.lpszTitle = "Select a directory for output:";
					bi.ulFlags = BIF_RETURNONLYFSDIRS;
					bi.lpfn = BrowseCallbackProc;
					bi.lParam = 0;
					idlist = SHBrowseForFolder( &bi );

					if (idlist) 
						SHGetPathFromIDList( idlist, szTemp );
					else
						strcpy(szTemp,"");

					SetDlgItemText(hwndDlg, IDC_EDITDIR, szTemp);
				}
				break;

				default:
					break;
        	  	}
	}
	return FALSE; 
}

/*
 *  About dialog procedures.
 */
BOOL CALLBACK AboutProc(HWND hwndDlg, UINT message, WPARAM wParam, LPARAM lParam) 
{
	switch (message){ 

		case WM_CLOSE:
			EndDialog(hwndDlg, TRUE);
			return TRUE;

		case WM_COMMAND: 
			switch (LOWORD(wParam)) {
				case IDC_BUTTON6:
					EndDialog(hwndDlg, TRUE);
					return TRUE;

        	  	}
	}
	return FALSE; 
}


/********************************** end of main.c **********************************/



